@@ -10,7 +10,6 @@
 
 // Qt
 #include <QFile>
-#include <QMap>
 #include <QPainter>
 #include <QPixmap>
 #include <QPixmapCache>
@@ -28,43 +27,35 @@
 class KMahjonggBackgroundPrivate
 {
 public:
-    KMahjonggBackgroundPrivate()
-        : w(1)
-        , h(1)
-        , graphicsLoaded(false)
-        , isPlain(false)
-        , isTiled(true)
-        , isSVG(false)
-    {
-    }
+    KMahjonggBackgroundPrivate() = default;
 
-    QMap<QString, QString> authorproperties;
-    QString pixmapCacheNameFromElementId(const QString & elementid);
+public:
+    QString name;
+    QString description;
+    QString authorName;
+    QString authorEmailAddress;
+
+    QString pixmapCacheNameFromElementId(const QString &elementid);
     QPixmap renderBG(short width, short height);
 
     QPixmap backgroundPixmap;
     QBrush backgroundBrush;
     QString filename;
     QString graphicspath;
-    short w;
-    short h;
+    short w = 1;
+    short h = 1;
 
     QSvgRenderer svg;
 
-    bool graphicsLoaded;
-    bool isPlain;
-    bool isTiled;
-    bool isSVG;
+    bool graphicsLoaded = false;
+    bool isPlain = false;
+    bool isTiled = true;
+    bool isSVG = false;
 };
 
 KMahjonggBackground::KMahjonggBackground()
-    : d(new KMahjonggBackgroundPrivate)
+    : d_ptr(new KMahjonggBackgroundPrivate)
 {
-    static bool _inited = false;
-    if (_inited) {
-        return;
-    }
-    _inited = true;
 }
 
 KMahjonggBackground::~KMahjonggBackground() = default;
@@ -72,9 +63,7 @@ KMahjonggBackground::~KMahjonggBackground() = default;
 bool KMahjonggBackground::loadDefault()
 {
     // Set default background here.
-    QLatin1String idx("egyptian.desktop");
-
-    QString bgPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kmahjongglib/backgrounds/") + idx);
+    const QString bgPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kmahjongglib/backgrounds/egyptian.desktop"));
     qCDebug(LIBKMAHJONGG_LOG) << "Inside LoadDefault(), located background at" << bgPath;
     if (bgPath.isEmpty()) {
         return false;
@@ -84,12 +73,14 @@ bool KMahjonggBackground::loadDefault()
 
 #define kBGVersionFormat 1
 
-bool KMahjonggBackground::load(const QString & file, short width, short height)
+bool KMahjonggBackground::load(const QString &file, short width, short height)
 {
-    //qCDebug(LIBKMAHJONGG_LOG) << "Background loading";
+    Q_D(KMahjonggBackground);
+
+    // qCDebug(LIBKMAHJONGG_LOG) << "Background loading";
     d->isSVG = false;
 
-    //qCDebug(LIBKMAHJONGG_LOG) << "Attempting to load .desktop at" << file;
+    // qCDebug(LIBKMAHJONGG_LOG) << "Attempting to load .desktop at" << file;
 
     // verify if it is a valid file first and if we can open it
     QFile bgfile(file);
@@ -99,25 +90,23 @@ bool KMahjonggBackground::load(const QString & file, short width, short height)
     bgfile.close();
 
     KConfig bgconfig(file, KConfig::SimpleConfig);
-    KConfigGroup group = bgconfig.group("KMahjonggBackground");
+    KConfigGroup group = bgconfig.group(QStringLiteral("KMahjonggBackground"));
 
-    d->authorproperties.insert(QStringLiteral("Name"), group.readEntry("Name")); // Returns translated data
-    d->authorproperties.insert(QStringLiteral("Author"), group.readEntry("Author"));
-    d->authorproperties.insert(QStringLiteral("Description"), group.readEntry("Description"));
-    d->authorproperties.insert(QStringLiteral("AuthorEmail"), group.readEntry("AuthorEmail"));
-    //The "Plain" key is set to 1 by the color_plain background.
     d->isPlain = group.readEntry("Plain", 0) != 0;
-    d->authorproperties.insert(QStringLiteral("Plain"), d->isPlain ? QStringLiteral("1") : QStringLiteral("0"));
+    d->name = group.readEntry("Name"); // Returns translated data
+    d->description = group.readEntry("Description");
+    d->authorName = group.readEntry("Author");
+    d->authorEmailAddress = group.readEntry("AuthorEmail");
 
-    //Version control
+    // Version control
     int bgversion = group.readEntry("VersionFormat", 0);
-    //Format is increased when we have incompatible changes, meaning that older clients are not able to use the remaining information safely
+    // Format is increased when we have incompatible changes, meaning that older clients are not able to use the remaining information safely
     if (bgversion > kBGVersionFormat) {
         return false;
     }
 
     if (d->isPlain) {
-        //qCDebug(LIBKMAHJONGG_LOG) << "Using plain background";
+        // qCDebug(LIBKMAHJONGG_LOG) << "Using plain background";
         d->graphicspath.clear();
         d->filename = file;
         return true;
@@ -149,6 +138,8 @@ bool KMahjonggBackground::load(const QString & file, short width, short height)
 
 bool KMahjonggBackground::loadGraphics()
 {
+    Q_D(KMahjonggBackground);
+
     if (d->graphicsLoaded || d->isPlain) {
         return true;
     }
@@ -157,7 +148,7 @@ bool KMahjonggBackground::loadGraphics()
     if (d->svg.isValid()) {
         d->isSVG = true;
     } else {
-        //qCDebug(LIBKMAHJONGG_LOG) << "could not load svg";
+        // qCDebug(LIBKMAHJONGG_LOG) << "could not load svg";
         return false;
     }
     return true;
@@ -165,7 +156,9 @@ bool KMahjonggBackground::loadGraphics()
 
 void KMahjonggBackground::sizeChanged(int newW, int newH)
 {
-    //in tiled mode we do not care about the whole field size
+    Q_D(KMahjonggBackground);
+
+    // in tiled mode we do not care about the whole field size
     if (d->isTiled || d->isPlain) {
         return;
     }
@@ -177,9 +170,9 @@ void KMahjonggBackground::sizeChanged(int newW, int newH)
     d->h = newH;
 }
 
-QString KMahjonggBackgroundPrivate::pixmapCacheNameFromElementId(const QString & elementid)
+QString KMahjonggBackgroundPrivate::pixmapCacheNameFromElementId(const QString &elementid)
 {
-    return authorproperties[QStringLiteral("Name")] + elementid + QStringLiteral("W%1H%2").arg(w).arg(h);
+    return name + elementid + QStringLiteral("W%1H%2").arg(w).arg(h);
 }
 
 QPixmap KMahjonggBackgroundPrivate::renderBG(short width, short height)
@@ -196,8 +189,10 @@ QPixmap KMahjonggBackgroundPrivate::renderBG(short width, short height)
     return qiRend;
 }
 
-QBrush & KMahjonggBackground::getBackground()
+QBrush &KMahjonggBackground::getBackground()
 {
+    Q_D(KMahjonggBackground);
+
     if (d->isPlain) {
         d->backgroundBrush = QBrush(QPixmap());
     } else {
@@ -212,10 +207,42 @@ QBrush & KMahjonggBackground::getBackground()
 
 QString KMahjonggBackground::path() const
 {
+    Q_D(const KMahjonggBackground);
+
     return d->filename;
 }
 
-QString KMahjonggBackground::authorProperty(const QString & key) const
+QString KMahjonggBackground::name() const
 {
-    return d->authorproperties[key];
+    Q_D(const KMahjonggBackground);
+
+    return d->name;
+}
+
+QString KMahjonggBackground::description() const
+{
+    Q_D(const KMahjonggBackground);
+
+    return d->description;
+}
+
+QString KMahjonggBackground::authorName() const
+{
+    Q_D(const KMahjonggBackground);
+
+    return d->authorName;
+}
+
+QString KMahjonggBackground::authorEmailAddress() const
+{
+    Q_D(const KMahjonggBackground);
+
+    return d->authorEmailAddress;
+}
+
+bool KMahjonggBackground::isPlain() const
+{
+    Q_D(const KMahjonggBackground);
+
+    return d->isPlain;
 }
