@@ -14,8 +14,6 @@
 // Qt
 #include <QFile>
 #include <QGuiApplication>
-#include <QImage>
-#include <QMap>
 #include <QPainter>
 #include <QPixmapCache>
 #include <QStandardPaths>
@@ -32,57 +30,45 @@
 class KMahjonggTilesetMetricsData
 {
 public:
-    short lvloffx; // used for 3D indentation, x value
-    short lvloffy; // used for 3D indentation, y value
-    short w; // tile width ( +border +shadow)
-    short h; // tile height ( +border +shadow)
-    short fw; // face width
-    short fh; // face height
+    short lvloffx = 0; // used for 3D indentation, x value
+    short lvloffy = 0; // used for 3D indentation, y value
+    short w = 0; // tile width ( +border +shadow)
+    short h = 0; // tile height ( +border +shadow)
+    short fw = 0; // face width
+    short fh = 0; // face height
 
-    KMahjonggTilesetMetricsData()
-        : lvloffx(0)
-        , lvloffy(0)
-        , w(0)
-        , h(0)
-        , fw(0)
-        , fh(0)
-    {
-    }
+    KMahjonggTilesetMetricsData() = default;
 };
 
 class KMahjonggTilesetPrivate
 {
 public:
-    KMahjonggTilesetPrivate()
-        : isSVG(false)
-        , graphicsLoaded(false)
-    {
-    }
+    KMahjonggTilesetPrivate() = default;
+
+public:
     QList<QString> elementIdTable;
-    QMap<QString, QString> authorproperties;
+
+    QString name;
+    QString description;
+    QString authorName;
+    QString authorEmailAddress;
 
     KMahjonggTilesetMetricsData originaldata;
     KMahjonggTilesetMetricsData scaleddata;
     QString filename; // cache the last file loaded to save reloading it
     QString graphicspath;
 
-    QSvgRenderer svg;
-    bool isSVG;
-    bool graphicsLoaded;
+    mutable QSvgRenderer svg; // render() is non-const
+    bool isSVG = false;
+    bool graphicsLoaded = false;
 };
 
 // ---------------------------------------------------------
 
 KMahjonggTileset::KMahjonggTileset()
-    : d(new KMahjonggTilesetPrivate)
+    : d_ptr(new KMahjonggTilesetPrivate)
 {
     buildElementIdTable();
-
-    static bool _inited = false;
-    if (_inited) {
-        return;
-    }
-    _inited = true;
 }
 
 // ---------------------------------------------------------
@@ -91,6 +77,8 @@ KMahjonggTileset::~KMahjonggTileset() = default;
 
 void KMahjonggTileset::updateScaleInfo(short tilew, short tileh)
 {
+    Q_D(KMahjonggTileset);
+
     d->scaleddata.w = tilew;
     d->scaleddata.h = tileh;
     double ratio = (static_cast<qreal>(d->scaleddata.w)) / (static_cast<qreal>(d->originaldata.w));
@@ -100,21 +88,23 @@ void KMahjonggTileset::updateScaleInfo(short tilew, short tileh)
     d->scaleddata.fh = static_cast<short>(d->originaldata.fh * ratio);
 }
 
-QSize KMahjonggTileset::preferredTileSize(const QSize & boardsize, int horizontalCells, int verticalCells)
+QSize KMahjonggTileset::preferredTileSize(QSize boardsize, int horizontalCells, int verticalCells) const
 {
-    //calculate our best tile size to fit the boardsize passed to us
+    Q_D(const KMahjonggTileset);
+
+    // calculate our best tile size to fit the boardsize passed to us
     qreal newtilew, newtileh, aspectratio;
     qreal bw = boardsize.width();
     qreal bh = boardsize.height();
 
-    //use tileface for calculation, with one complete tile in the sum for extra margin
+    // use tileface for calculation, with one complete tile in the sum for extra margin
     qreal fullh = (d->originaldata.fh * verticalCells) + d->originaldata.h;
     qreal fullw = (d->originaldata.fw * horizontalCells) + d->originaldata.w;
     qreal floatw = d->originaldata.w;
     qreal floath = d->originaldata.h;
 
     if ((fullw / fullh) > (bw / bh)) {
-        //space will be left on height, use width as limit
+        // space will be left on height, use width as limit
         aspectratio = bw / fullw;
     } else {
         aspectratio = bh / fullh;
@@ -126,9 +116,9 @@ QSize KMahjonggTileset::preferredTileSize(const QSize & boardsize, int horizonta
 
 bool KMahjonggTileset::loadDefault()
 {
-    QString idx = QStringLiteral("default.desktop");
+    Q_D(KMahjonggTileset);
 
-    QString tilesetPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kmahjongglib/tilesets/") + idx);
+    const QString tilesetPath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kmahjongglib/tilesets/default.desktop"));
     qCDebug(LIBKMAHJONGG_LOG) << "Inside LoadDefault(), located path at" << tilesetPath;
     if (tilesetPath.isEmpty()) {
         return false;
@@ -136,74 +126,114 @@ bool KMahjonggTileset::loadDefault()
     return loadTileset(tilesetPath);
 }
 
-QString KMahjonggTileset::authorProperty(const QString & key) const
+QString KMahjonggTileset::name() const
 {
-    return d->authorproperties[key];
+    Q_D(const KMahjonggTileset);
+
+    return d->name;
+}
+
+QString KMahjonggTileset::description() const
+{
+    Q_D(const KMahjonggTileset);
+
+    return d->description;
+}
+
+QString KMahjonggTileset::authorName() const
+{
+    Q_D(const KMahjonggTileset);
+
+    return d->authorName;
+}
+
+QString KMahjonggTileset::authorEmailAddress() const
+{
+    Q_D(const KMahjonggTileset);
+
+    return d->authorEmailAddress;
 }
 
 short KMahjonggTileset::width() const
 {
+    Q_D(const KMahjonggTileset);
+
     return d->scaleddata.w;
 }
 
 short KMahjonggTileset::height() const
 {
+    Q_D(const KMahjonggTileset);
+
     return d->scaleddata.h;
 }
 
 short KMahjonggTileset::levelOffsetX() const
 {
+    Q_D(const KMahjonggTileset);
+
     return d->scaleddata.lvloffx;
 }
 
 short KMahjonggTileset::levelOffsetY() const
 {
+    Q_D(const KMahjonggTileset);
+
     return d->scaleddata.lvloffy;
 }
 
 short KMahjonggTileset::qWidth() const
 {
+    Q_D(const KMahjonggTileset);
+
     return static_cast<short>(d->scaleddata.fw / 2.0);
 }
 
 short KMahjonggTileset::qHeight() const
 {
+    Q_D(const KMahjonggTileset);
+
     return static_cast<short>(d->scaleddata.fh / 2.0);
 }
 
 QString KMahjonggTileset::path() const
 {
+    Q_D(const KMahjonggTileset);
+
     return d->filename;
 }
 
 #define kTilesetVersionFormat 1
 
 // ---------------------------------------------------------
-bool KMahjonggTileset::loadTileset(const QString & tilesetPath)
+bool KMahjonggTileset::loadTileset(const QString &tilesetPath)
 {
-    //qCDebug(LIBKMAHJONGG_LOG) << "Attempting to load .desktop at" << tilesetPath;
+    Q_D(KMahjonggTileset);
 
-    //clear our properties map
-    d->authorproperties.clear();
+    // qCDebug(LIBKMAHJONGG_LOG) << "Attempting to load .desktop at" << tilesetPath;
 
     // verify if it is a valid file first and if we can open it
     QFile tilesetfile(tilesetPath);
     if (!tilesetfile.open(QIODevice::ReadOnly)) {
+        d->name.clear();
+        d->description.clear();
+        d->authorName.clear();
+        d->authorEmailAddress.clear();
         return false;
     }
     tilesetfile.close();
 
     KConfig tileconfig(tilesetPath, KConfig::SimpleConfig);
-    KConfigGroup group = tileconfig.group("KMahjonggTileset");
+    KConfigGroup group = tileconfig.group(QStringLiteral("KMahjonggTileset"));
 
-    d->authorproperties.insert(QStringLiteral("Name"), group.readEntry("Name")); // Returns translated data
-    d->authorproperties.insert(QStringLiteral("Author"), group.readEntry("Author"));
-    d->authorproperties.insert(QStringLiteral("Description"), group.readEntry("Description"));
-    d->authorproperties.insert(QStringLiteral("AuthorEmail"), group.readEntry("AuthorEmail"));
+    d->name = group.readEntry("Name"); // Returns translated data
+    d->description = group.readEntry("Description");
+    d->authorName = group.readEntry("Author");
+    d->authorEmailAddress = group.readEntry("AuthorEmail");
 
-    //Version control
+    // Version control
     int tileversion = group.readEntry("VersionFormat", 0);
-    //Format is increased when we have incompatible changes, meaning that older clients are not able to use the remaining information safely
+    // Format is increased when we have incompatible changes, meaning that older clients are not able to use the remaining information safely
     if (tileversion > kTilesetVersionFormat) {
         return false;
     }
@@ -211,9 +241,9 @@ bool KMahjonggTileset::loadTileset(const QString & tilesetPath)
     QString graphName = group.readEntry("FileName");
 
     d->graphicspath = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kmahjongglib/tilesets/") + graphName);
-    //qCDebug(LIBKMAHJONGG_LOG) << "Using tileset at" << d->graphicspath;
+    // qCDebug(LIBKMAHJONGG_LOG) << "Using tileset at" << d->graphicspath;
 
-    //only SVG for now
+    // only SVG for now
     d->isSVG = true;
     if (d->graphicspath.isEmpty()) {
         return false;
@@ -226,7 +256,7 @@ bool KMahjonggTileset::loadTileset(const QString & tilesetPath)
     d->originaldata.lvloffx = group.readEntry("LevelOffsetX", 10);
     d->originaldata.lvloffy = group.readEntry("LevelOffsetY", 10);
 
-    //client application needs to call loadGraphics()
+    // client application needs to call loadGraphics()
     d->graphicsLoaded = false;
     d->filename = tilesetPath;
 
@@ -236,14 +266,16 @@ bool KMahjonggTileset::loadTileset(const QString & tilesetPath)
 // ---------------------------------------------------------
 bool KMahjonggTileset::loadGraphics()
 {
+    Q_D(KMahjonggTileset);
+
     if (d->graphicsLoaded) {
         return true;
     }
     if (d->isSVG) {
-        //really?
+        // really?
         d->svg.load(d->graphicspath);
         if (d->svg.isValid()) {
-            //invalidate our global cache
+            // invalidate our global cache
             QPixmapCache::clear();
             d->graphicsLoaded = true;
             reloadTileset(QSize(d->originaldata.w, d->originaldata.h));
@@ -251,7 +283,7 @@ bool KMahjonggTileset::loadGraphics()
             return false;
         }
     } else {
-        //TODO add support for png??
+        // TODO add support for png??
         return false;
     }
 
@@ -259,8 +291,10 @@ bool KMahjonggTileset::loadGraphics()
 }
 
 // ---------------------------------------------------------
-bool KMahjonggTileset::reloadTileset(const QSize & newTilesize)
+bool KMahjonggTileset::reloadTileset(QSize newTilesize)
 {
+    Q_D(KMahjonggTileset);
+
     if (QSize(d->scaleddata.w, d->scaleddata.h) == newTilesize) {
         return false;
     }
@@ -268,12 +302,12 @@ bool KMahjonggTileset::reloadTileset(const QSize & newTilesize)
     if (d->isSVG) {
         if (d->svg.isValid()) {
             updateScaleInfo(newTilesize.width(), newTilesize.height());
-            //rendering will be done when needed, automatically using the global cache
+            // rendering will be done when needed, automatically using the global cache
         } else {
             return false;
         }
     } else {
-        //TODO add support for png???
+        // TODO add support for png???
         return false;
     }
 
@@ -282,96 +316,119 @@ bool KMahjonggTileset::reloadTileset(const QSize & newTilesize)
 
 void KMahjonggTileset::buildElementIdTable()
 {
-    //Build a list for faster lookup of element ids, mapped to the enumeration used by GameData and BoardWidget
-    //Unselected tiles
-    for (short idx = 1; idx <= 4; idx++) {
+    Q_D(KMahjonggTileset);
+
+    constexpr int tileCount = 4;
+    constexpr int characterCount = 9;
+    constexpr int bambooCount = 9;
+    constexpr int rodCount = 9;
+    constexpr int seasonCount = 4;
+    constexpr int windCount = 4;
+    constexpr int dragonCount = 3;
+    constexpr int flowerCount = 4;
+
+    d->elementIdTable.reserve(2 * tileCount + characterCount + bambooCount + rodCount + seasonCount + windCount + dragonCount + flowerCount);
+
+    // Build a list for faster lookup of element ids, mapped to the enumeration used by GameData and BoardWidget
+    // Unselected tiles
+    for (short idx = 1; idx <= tileCount; idx++) {
         d->elementIdTable.append(QStringLiteral("TILE_%1").arg(idx));
     }
-    //Selected tiles
-    for (short idx = 1; idx <= 4; idx++) {
+    // Selected tiles
+    for (short idx = 1; idx <= tileCount; idx++) {
         d->elementIdTable.append(QStringLiteral("TILE_%1_SEL").arg(idx));
     }
-    //now faces
-    for (short idx = 1; idx <= 9; idx++) {
+    // now faces
+    for (short idx = 1; idx <= characterCount; idx++) {
         d->elementIdTable.append(QStringLiteral("CHARACTER_%1").arg(idx));
     }
-    for (short idx = 1; idx <= 9; idx++) {
+    for (short idx = 1; idx <= bambooCount; idx++) {
         d->elementIdTable.append(QStringLiteral("BAMBOO_%1").arg(idx));
     }
-    for (short idx = 1; idx <= 9; idx++) {
+    for (short idx = 1; idx <= rodCount; idx++) {
         d->elementIdTable.append(QStringLiteral("ROD_%1").arg(idx));
     }
-    for (short idx = 1; idx <= 4; idx++) {
+    for (short idx = 1; idx <= seasonCount; idx++) {
         d->elementIdTable.append(QStringLiteral("SEASON_%1").arg(idx));
     }
-    for (short idx = 1; idx <= 4; idx++) {
+    for (short idx = 1; idx <= windCount; idx++) {
         d->elementIdTable.append(QStringLiteral("WIND_%1").arg(idx));
     }
-    for (short idx = 1; idx <= 3; idx++) {
+    for (short idx = 1; idx <= dragonCount; idx++) {
         d->elementIdTable.append(QStringLiteral("DRAGON_%1").arg(idx));
     }
-    for (short idx = 1; idx <= 4; idx++) {
+    for (short idx = 1; idx <= flowerCount; idx++) {
         d->elementIdTable.append(QStringLiteral("FLOWER_%1").arg(idx));
     }
 }
 
-QString KMahjonggTileset::pixmapCacheNameFromElementId(const QString & elementid)
+QString KMahjonggTileset::pixmapCacheNameFromElementId(const QString &elementid) const
 {
-    return authorProperty(QStringLiteral("Name")) + elementid + QStringLiteral("W%1H%2").arg(d->scaleddata.w).arg(d->scaleddata.h);
+    Q_D(const KMahjonggTileset);
+
+    return d->name + elementid + QStringLiteral("W%1H%2").arg(d->scaleddata.w).arg(d->scaleddata.h);
 }
 
-QPixmap KMahjonggTileset::renderElement(short width, short height, const QString & elementid)
+QPixmap KMahjonggTileset::renderElement(short width, short height, const QString &elementid) const
 {
-    //qCDebug(LIBKMAHJONGG_LOG) << "render element" << elementid << width << height;
+    Q_D(const KMahjonggTileset);
+
+    // qCDebug(LIBKMAHJONGG_LOG) << "render element" << elementid << width << height;
     const qreal dpr = qApp->devicePixelRatio();
     width = width * dpr;
     height = height * dpr;
-    QImage qiRend(QSize(width, height), QImage::Format_ARGB32_Premultiplied);
-    qiRend.fill(0);
+    QPixmap qiRend(width, height);
+    qiRend.fill(Qt::transparent);
 
     if (d->svg.isValid()) {
         QPainter p(&qiRend);
         d->svg.render(&p, elementid);
     }
     qiRend.setDevicePixelRatio(dpr);
-    return QPixmap::fromImage(qiRend);
+    return qiRend;
 }
 
-QPixmap KMahjonggTileset::selectedTile(int num)
+QPixmap KMahjonggTileset::selectedTile(int num) const
 {
+    Q_D(const KMahjonggTileset);
+
     QPixmap pm;
-    QString elemId = d->elementIdTable.at(num + 4); //selected offset in our idtable;
+    QString elemId = d->elementIdTable.at(num + 4); // selected offset in our idtable;
     if (!QPixmapCache::find(pixmapCacheNameFromElementId(elemId), &pm)) {
-        //use tile size
+        // use tile size
         pm = renderElement(d->scaleddata.w, d->scaleddata.h, elemId);
         QPixmapCache::insert(pixmapCacheNameFromElementId(elemId), pm);
     }
     return pm;
 }
 
-QPixmap KMahjonggTileset::unselectedTile(int num)
+QPixmap KMahjonggTileset::unselectedTile(int num) const
 {
+    Q_D(const KMahjonggTileset);
+
     QPixmap pm;
     QString elemId = d->elementIdTable.at(num);
     if (!QPixmapCache::find(pixmapCacheNameFromElementId(elemId), &pm)) {
-        //use tile size
+        // use tile size
         pm = renderElement(d->scaleddata.w, d->scaleddata.h, elemId);
         QPixmapCache::insert(pixmapCacheNameFromElementId(elemId), pm);
     }
     return pm;
 }
 
-QPixmap KMahjonggTileset::tileface(int num)
+QPixmap KMahjonggTileset::tileface(int num) const
 {
+    Q_D(const KMahjonggTileset);
+
     QPixmap pm;
     if ((num + 8) >= d->elementIdTable.count()) {
-        //qCDebug(LIBKMAHJONGG_LOG) << "Client asked for invalid tileface id";
+        // qCDebug(LIBKMAHJONGG_LOG) << "Client asked for invalid tileface id";
         return pm;
     }
 
-    QString elemId = d->elementIdTable.at(num + 8); //tileface offset in our idtable;
+    QString elemId = d->elementIdTable.at(num + 8); // tileface offset in our idtable;
     if (!QPixmapCache::find(pixmapCacheNameFromElementId(elemId), &pm)) {
-        //use face size
+        // use face size
         pm = renderElement(d->scaleddata.fw, d->scaleddata.fh, elemId);
         QPixmapCache::insert(pixmapCacheNameFromElementId(elemId), pm);
     }
